function [Final_crossings, finClab, finCidx] = Quick_cross( el,embed_scheme,min_scheme,seed,bigface_depth,origclabel,origcidx,x,y,verbosity )
% An implementation of https://arxiv.org/abs/1804.09900
%
% Copyright (C) 2018 Michael Haythorpe, Alex Newcombe.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <https://www.gnu.org/licenses/>.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% 
% Please read the QC_C_README and QC_MATLAB_README and try the C version first, then the options 
% here will make more sense.
% 
% Inputs:
% 
% el: edge list of the graph to solve. Alternatively graph_el.m provides 
% various pre-constructed graph families. See that file for more info.
% 
% embed_scheme: Initial embedding scheme options:
% 
% 1 : Kamada Kawai spring model drawing.
% 2 : embed onto circle.
% 3 : planar build embedding - build upon a chordless cycle, one vertex at 
%     a time.
% 4 : specify a crossing order list corresponding to a planarisation of a 
%     drawing of the graph (see origclabel and origcidx for appropriate 
%     format).
% 5 : specify coordinates in the plane corresponding to vertex locations. 
%     The edges will then be drawn as straight lines and the graph will be 
%     solved from there (see 'x and y' for appropriate format).
% 
% min_scheme: Minimising scheme options:
% 
% 1 : use bigface scheme.
% 2 : use improvements as soon as they are found.
% 3 : search all vertices for best improvement before proceeding.
% 
% seed: random seed, if zero then no randomisation occurs.
% 
% bigface_depth: only used if min_scheme=1. Can become very slow if larger 
% than 3.
% 
% origclabel and origcidx: only used if embed_scheme=4. The crossings on
% edge i must be in the format: origclabel(origcidx(i)+1:origcidx(i+1)). An
% example is provided in QC_MATLAB_CR_EXAMPLE.txt.
% 
% x and y: only used if embed_scheme=5. Both are nx1 vectors, with the
% coordinate of vertex i on row i.
%
% verbosity: Style of on-the-fly information displayed during the
% heuristic. If zero then no information is displayed, if one then
% crossings at each iteration is displayed.
% 
% Outputs:
% 
% Final_crossings: Number of crossings obtained in the final drawing.
% 
% finClab: Crossing order list obtained in the final drawing.
% 
% finCidx: Crossing order index list for the final drawing.
% 
% Note the final crossing order of edge i is obtained using 
% finClab(finCidx(i)+1:finCidx(i+1)).

draw=0;
if seed>0
    rng(seed);
end
if size(el,2)~=2
    N=size(el,1);
    M=sum(sum(el))/2;
    el=adj_to_el(el);
else
    N=max(max(el));
    M=size(el,1);
end
deglist=zeros(N,1);
vadj=zeros(2*M,1);
eadj=zeros(2*M,1);
vidx=zeros(N+1,1);
for i=1:M
    deglist(el(i,1))=deglist(el(i,1))+1;
    deglist(el(i,2))=deglist(el(i,2))+1;
end
for i=1:N
    vidx(i+1)=vidx(i)+deglist(i);
end
degidx = zeros(N,1);
for i=1:M
    v1 = el(i,1);
    v2 = el(i,2);
    degidx(v1)=degidx(v1)+1;
    degidx(v2)=degidx(v2)+1;
    vadj(vidx(v1)+degidx(v1))=v2;
    vadj(vidx(v2)+degidx(v2))=v1;
    eadj(vidx(v1)+degidx(v1))=i;
    eadj(vidx(v2)+degidx(v2))=i;
end
if embed_scheme==4
    [finClab,finCidx,Final_crossings,planar]=embedding_given_crossings(el,origclabel,origcidx,2,min_scheme,bigface_depth,verbosity);
    if planar==0
        disp('given crossings do not define a planar planarization');
        return;
    end
    return;
end
einvmapdel1 = (1:M)';
vinvmapdel1 = (1:N)';
finClab1 = zeros(M,1);
fincCcnt = zeros(M,1);
finCidx1 = zeros(M,1);
fcnt = 0;
origM = M;
origN = N;
% remove degree one vertices, as they do not effect cr(G)
checkdel=1;
% changed=0;
while checkdel
    checkdel=0;
    for i=1:N
        if deglist(i)==1
            %             changed=1;
            checkdel=1;
            v1=vadj(vidx(i)+1);
            e1 = eadj(vidx(i)+1);
            f1=find(vadj(vidx(v1)+1:vidx(v1+1))==i);
            for j=1:vidx(N+1)
                if vadj(j)>i
                    vadj(j)=vadj(j)-1;
                end
                if eadj(j)>e1
                    eadj(j)=eadj(j)-1;
                end
            end
            vadj(vidx(v1)+f1)=[];
            eadj(vidx(v1)+f1)=[];
            deglist(v1)=deglist(v1)-1;
            vidx(v1+1:N+1)=vidx(v1+1:N+1)-1;
            vadj=[vadj(1:vidx(i));vadj(vidx(i)+2:end)];
            eadj=[eadj(1:vidx(i));eadj(vidx(i)+2:end)];
            einvmapdel1(e1) = [];
            vinvmapdel1(i)=[];
            vidx=[vidx(1:i);vidx(i+2:end)-1];
            deglist=[deglist(1:i-1);deglist(i+1:N)];
            N=N-1;
            M=M-1;
            break
        end
    end
end
%find connected components
ccnt=0;
ctcnt=0;
ccidx=0;
clst=[];
visited=zeros(N,1);
for i=1:N
    if visited(i)==0
        ctcnt=ctcnt+1;
        ccnt=ccnt+1;
        clst(ccnt,1)=i;
        stack=zeros(N,1);
        stackpos=0;
        stackmax=1;
        stack(1)=i;
        visited(i)=1;
        while stackpos<stackmax
            stackpos=stackpos+1;
            currv=stack(stackpos);
            for j=vidx(currv)+1:vidx(currv+1)
                if visited(vadj(j))==0
                    visited(vadj(j))=1;
                    ccnt=ccnt+1;
                    clst(ccnt,1)=vadj(j);
                    stackmax=stackmax+1;
                    stack(stackmax)=vadj(j);
                end
            end
        end
        ccidx(ctcnt+1)=ccnt;
    end
end
Final_crossings=0;
for z=1:ctcnt
    %%update only need to count components of size 4 or more, else cr(component)=0.
    if ccidx(z+1)-ccidx(z)<5
        continue;
    end
    visited=zeros(N,1);
    LP=zeros(N,1);
    Tcheck=zeros(M,1);
    T=zeros(M,1);
    tcount=0;
    p=zeros(N,1);
    clst2=zeros(2*N,1);
    ccidx2=zeros(N,1);
    ccnt=0;
    acompcnt=0;
    ecnt = 0;
    elst = zeros(M,1);
    eidx = zeros(ccidx(z+1)-ccidx(z),1);
    %find maximal two connected components withing this connected
    %component.
    [~,~,~,~,~,~,~,clst2,~,ccidx2,acompcnt,~,elst,eidx] = UDFS(clst(ccidx(z)+1),1,visited,LP,Tcheck,T,tcount,p,el,vidx,vadj,clst2,ccnt,eadj,origN,ccidx2,acompcnt,einvmapdel1,ecnt,elst,eidx);
    if acompcnt>1
        for i=1:acompcnt
            relcnt=ccidx2(i+1)-ccidx2(i);
            if relcnt<5
                continue; %no contributions
            end
            Mcnt = eidx(i+1)-eidx(i);
            currvadj=zeros(2*Mcnt,1);
            curreadj=zeros(2*Mcnt,1);
            currvidx=zeros(relcnt+1,1);
            currdeg=zeros(relcnt,1);
            einvmaptwo = elst(eidx(i)+1:eidx(i+1));
            vinvmaptwo = clst2(ccidx2(i)+1:ccidx2(i+1));
            vmaptwo=zeros(origN,1);
            for j=1:relcnt
                vmaptwo(vinvmaptwo(j))=j;
            end
            for j=eidx(i)+1:eidx(i+1)
                currdeg(vmaptwo(el(elst(j),1)))=currdeg(vmaptwo(el(elst(j),1)))+1;
                currdeg(vmaptwo(el(elst(j),2)))=currdeg(vmaptwo(el(elst(j),2)))+1;
            end
            for j=1:relcnt
                currvidx(j+1)=currvidx(j)+currdeg(j);
            end
            degidx = zeros(relcnt,1);
            jc = 0;
            for j=eidx(i)+1:eidx(i+1)
                jc = jc + 1;
                v1 = vmaptwo(el(elst(j),1));
                v2 = vmaptwo(el(elst(j),2));
                degidx(v1)=degidx(v1)+1;
                degidx(v2)=degidx(v2)+1;
                currvadj(currvidx(v1)+degidx(v1))=v2;
                currvadj(currvidx(v2)+degidx(v2))=v1;
                curreadj(currvidx(v1)+degidx(v1))=jc;
                curreadj(currvidx(v2)+degidx(v2))=jc;
            end
            %deg 2 reductions
            t=1;
            while t==1
                t=0;
                if relcnt<5
                    break %then k2
                end
                for j=1:relcnt
                    if currdeg(j)==2
                        t=1;
                        v1=currvadj(currvidx(j)+1);
                        v2=currvadj(currvidx(j)+2);
                        if currdeg(v2)>2
                            v1=currvadj(currvidx(j)+2);
                            v2=currvadj(currvidx(j)+1);
                        end
                        f2=find(currvadj(currvidx(v1)+1:currvidx(v1+1))==j);
                        if sum(currvadj(currvidx(v1)+1:currvidx(v1+1))==v2)==0
                            currvadj(currvidx(v1)+f2)=v2;
                            e1 = curreadj(currvidx(v1)+f2);
                        else
                            %dont reduce because of multi-edge
                            t=0;
                            continue;
                        end
                        f2=find(currvadj(currvidx(v2)+1:currvidx(v2+1))==j);
                        currvadj(currvidx(v2)+f2)=v1;
                        e2 = curreadj(currvidx(v2)+f2);
                        curreadj(currvidx(v2)+f2)=e1;
                        
                        currvadj=[currvadj(1:currvidx(j));currvadj(currvidx(j+1)+1:end)];
                        curreadj=[curreadj(1:currvidx(j));curreadj(currvidx(j+1)+1:end)];
                        currvidx(j+1:relcnt+1)=[currvidx(j+2:relcnt+1)-2;0];                            
                        currdeg(j:relcnt)=[currdeg(j+1:relcnt);0];
                        einvmaptwo(e2:Mcnt) = [einvmaptwo(e2+1:Mcnt);0];
                        vinvmaptwo(j:relcnt) = [vinvmaptwo(j+1:relcnt);0];
                        for k=1:origN
                            if vmaptwo(k)==j
                                vmaptwo(k)=v2;
                            end
                        end
                        for k=1:origN
                            if vmaptwo(k)>j
                                vmaptwo(k)=vmaptwo(k)-1;
                            end
                        end
                        for k=1:currvidx(relcnt)
                            if currvadj(k)>j
                                currvadj(k)=currvadj(k)-1;
                            end
                            if curreadj(k)>e2
                                curreadj(k)=curreadj(k)-1;
                            end
                        end
                        relcnt=relcnt-1;
                        Mcnt=Mcnt-1;
                        break
                    end
                end
            end
            if relcnt<5
                continue;
            end
            Sixidx=zeros(Mcnt,6);
            for j=1:relcnt
                for k=currvidx(j)+1:currvidx(j+1)
                    if j<currvadj(k)
                        Sixidx(curreadj(k),[1 2])=[j,currvadj(k)];
                    else
                        Sixidx(curreadj(k),[1 2])=[currvadj(k),j];
                    end
                end
            end
            
            incidente=zeros(relcnt,1);
            for j=1:Mcnt
                if incidente(Sixidx(j,1))==0
                    incidente(Sixidx(j,1))=j;
                end
                if incidente(Sixidx(j,2))==0
                    incidente(Sixidx(j,2))=j;
                end
            end
            currdeg=currdeg(1:relcnt);
            currvadj=currvadj(1:currvidx(relcnt+1));
            currvidx=currvidx(1:relcnt+1);
            curreadj=curreadj(1:currvidx(relcnt+1));
	    Nbeforeembed=relcnt;
	    Mbeforeembed=Mcnt;
            if embed_scheme==1
                [Sixidx,Clabel,Dlabel,Cidx] = get_spring_data(Sixidx(:,1:2),currdeg,currvadj,currvidx,0,[],[],seed);
            elseif embed_scheme==2
                [Clabel,Cidx,Sixidx,Dlabel ] = embed_circ_2( relcnt,Mcnt,seed,Sixidx,currvadj,curreadj,currvidx,draw );
                
            elseif embed_scheme==3

                [ Sixidx,incidente,Clabel,Dlabel,Cidx,relcnt,Mcnt,currdeg ]  = embed_plane_build( relcnt,Mcnt,Sixidx,currdeg,seed,draw,incidente,currvadj,currvidx,curreadj );
                if Sixidx(1,3)==0
                    %then acyclic
                    continue;
                end
            elseif embed_scheme==5
                currx=zeros(relcnt,1);
                curry=zeros(relcnt,1);
                for j=1:relcnt
                    currx(j)=x(vinvmaptwo(j));
                    curry(j)=y(vinvmaptwo(j));
                end
                [Sixidx,Clabel,Dlabel,Cidx] = get_spring_data(Sixidx(:,1:2),currdeg,currvadj,currvidx,1,currx,curry,0);
            elseif embed_scheme==6
                
                [ Sixidx,incidente,Clabel,Dlabel,Cidx,relcnt,Mcnt,currdeg ]  = embed_plane_build_incremental( relcnt,Mcnt,Sixidx,currdeg,seed,draw,incidente,currvadj,currvidx,curreadj );
                if Sixidx(1,3)==0
                    %then acyclic
                    continue;
                end
            end
            
            current_crossings=Cidx(Mcnt+1)/2;
            incidente=zeros(Mcnt,1);
            for j=1:Mcnt
                if incidente(Sixidx(j,1))==0
                    incidente(Sixidx(j,1))=j;
                end
                if incidente(Sixidx(j,2))==0
                    incidente(Sixidx(j,2))=j;
                end
            end
           
            [ Final_crossings2,Clabel,Cidx,Mcnt ] = Quick_cross_main_loop( incidente,Sixidx,Clabel,Dlabel,Cidx,relcnt,Mcnt,currdeg,current_crossings,min_scheme,bigface_depth,Nbeforeembed,verbosity);
%             [planar3]=CPP_runner2(Sixidx(:,[1 2]),Clabel,Cidx,0)
            
            if Mbeforeembed ~= Mcnt
                disp('undo sub not back to original M 1')
                pause
            end
            Final_crossings=Final_crossings+Final_crossings2;
            for j=1:Mcnt
                temph=zeros(Cidx(j+1)-Cidx(j),1);
                temph2=zeros(Cidx(j+1)-Cidx(j),1);
                counth=0;
                for k=Cidx(j)+1:Cidx(j+1)
                    counth=counth+1;
                    temph(counth)=einvmaptwo(Clabel(k));
                    temph2(counth)=einvmaptwo(j);
                    fincCcnt(einvmaptwo(j)) = fincCcnt(einvmaptwo(j)) + 1;
                end
                if vinvmaptwo(Sixidx(j,1))==el(einvmaptwo(j),1) || vinvmaptwo(Sixidx(j,2))==el(einvmaptwo(j),2)%this vinvmap includes before del1.
                    %then same dir
                    finClab1(fcnt+1:fcnt+counth)=temph;
                    finCidx1(fcnt+1:fcnt+counth)=temph2;                    
                elseif vinvmaptwo(Sixidx(j,2))==el(einvmaptwo(j),1) || vinvmaptwo(Sixidx(j,1))==el(einvmaptwo(j),2)
                    finClab1(fcnt+1:fcnt+counth)=temph(counth:-1:1);
                    finCidx1(fcnt+1:fcnt+counth)=temph2(counth:-1:1);                     
                else
                    disp('biconn new map error1');
                    pause
                end
                fcnt=fcnt+counth;
            end
            
        end
        
    else
        %run on just the current component
        if ctcnt>1
            newN=ccidx(z+1)-ccidx(z);
            relchk=clst(ccidx(z)+1:ccidx(z+1));
            vmaptwo=zeros(origN,1);
            for j=1:newN
                vmaptwo(relchk(j))=j;
            end
            rinvmap=zeros(max(relchk),1);
            currdeg=deglist(relchk);
            newM=sum(currdeg)/2;
            currvadj=zeros(2*newM,1);
            curreadj=zeros(2*newM,1);
            currvidx=zeros(newN+1,1);
            echeck = zeros(M,1);
            Mcnt = 0;
            for j=1:newN
                currvidx(j+1)=currvidx(j)+deglist(relchk(j));
                rinvmap(relchk(j))=j;
            end
            einvmaptwo = zeros(newM,1);
            for j=1:newN
                currvadj(currvidx(j)+1:currvidx(j+1))=rinvmap(vadj(vidx(relchk(j))+1:vidx(relchk(j)+1)));
                for k=vidx(relchk(j))+1:vidx(relchk(j)+1)
                    if echeck(eadj(k))==0
                        Mcnt = Mcnt+1;
                        echeck(eadj(k))=Mcnt;
                        einvmaptwo(Mcnt) = eadj(k);
                    end
                end
                curreadj(currvidx(j)+1:currvidx(j+1))=echeck(eadj(vidx(relchk(j))+1:vidx(relchk(j)+1)));
            end
            t=1;
            while t==1
                t=0;
                if newN<5
                    break
                end
                for j=1:newN
                    if currdeg(j)==2
                        t=1;
                        v1=currvadj(currvidx(j)+1);
                        v2=currvadj(currvidx(j)+2);
                        if currdeg(v2)>2
                            v1=currvadj(currvidx(j)+2);
                            v2=currvadj(currvidx(j)+1);
                        end
                        f2=find(currvadj(currvidx(v1)+1:currvidx(v1+1))==j);
                        if sum(currvadj(currvidx(v1)+1:currvidx(v1+1))==v2)==0
                            currvadj(currvidx(v1)+f2)=v2;
                            e1 = curreadj(currvidx(v1)+f2);
                        else
                            %dont reduce because of multi-edge
                            t=0;
                            continue;
                        end
                        f2=find(currvadj(currvidx(v2)+1:currvidx(v2+1))==j);
                        currvadj(currvidx(v2)+f2)=v1;
                        e2 = curreadj(currvidx(v2)+f2);
                        curreadj(currvidx(v2)+f2)=e1;
                        currvadj=[currvadj(1:currvidx(j));currvadj(currvidx(j+1)+1:end)];
                        curreadj=[curreadj(1:currvidx(j));curreadj(currvidx(j+1)+1:end)];
                        currvidx(j+1:newN+1)=[currvidx(j+2:newN+1)-2;0];
                        currdeg(j:newN)=[currdeg(j+1:newN);0];
                        einvmaptwo(e2:newM) = [einvmaptwo(e2+1:newM);0]; %e1 still maps to a deg2 edge in orig.
                        relchk(j:newN)=[relchk(j+1:newN);0];
                        for k=1:newN
                            if(vmaptwo(k)==j)
                                vmaptwo(k)=v2;
                            end
                        end
                        for k=1:newN
                            if(vmaptwo(k)>j)
                                vmaptwo(k)=vmaptwo(k)-1;
                            end
                        end
                        for k=1:currvidx(newN)
                            if currvadj(k)>j
                                currvadj(k)=currvadj(k)-1;
                            end
                            if curreadj(k)>e2
                                curreadj(k)=curreadj(k)-1;
                            end
                        end
                        newN=newN-1;
                        newM=newM-1;
                        break
                    end
                end
            end
            if newN<5
                continue;
            end
            Sixidx=zeros(newM,6);
            for j=1:newN
                for k=currvidx(j)+1:currvidx(j+1)
                    if j<currvadj(k)
                        Sixidx(curreadj(k),[1 2])=[j,currvadj(k)];
                    else
                        Sixidx(curreadj(k),[1 2])=[currvadj(k),j];
                    end
                end
            end
            incidente=zeros(newN,1);
            for i=1:newM
                if incidente(Sixidx(i,1))==0
                    incidente(Sixidx(i,1))=i;
                end
                if incidente(Sixidx(i,2))==0
                    incidente(Sixidx(i,2))=i;
                end
            end
Nbeforeembed=newN;
Mbeforeembed=newM;
            if embed_scheme==1
                [Sixidx,Clabel,Dlabel,Cidx] = get_spring_data(Sixidx(:,1:2),currdeg,currvadj,currvidx,0,[],[],seed);               
            elseif embed_scheme==2
                [Clabel,Cidx,Sixidx,Dlabel ] = embed_circ_2( newN,newM,seed,Sixidx,currvadj,curreadj,currvidx,draw );
            elseif embed_scheme==3
                [ Sixidx,incidente,Clabel,Dlabel,Cidx,newN,newM,currdeg ]  = embed_plane_build( newN,newM,Sixidx,currdeg,seed,draw,incidente,currvadj,currvidx,curreadj );
                if Sixidx(1,3)==0
                    %then acyclic
                    continue;
                end
            elseif embed_scheme==5
                currx=zeros(newN,1);
                curry=zeros(newN,1);
                for j=1:newN
                    currx(j)=x(vinvmaptwo(vinvmapdel1(j)));
                    curry(j)=y(vinvmaptwo(vinvmapdel1(j)));
                end
                [Sixidx,Clabel,Dlabel,Cidx] = get_spring_data(Sixidx(:,1:2),currdeg,currvadj,currvidx,1,currx,curry,0);
            elseif embed_scheme==6
                [ Sixidx,incidente,Clabel,Dlabel,Cidx,newN,newM,currdeg ]  = embed_plane_build_incremental( newN,newM,Sixidx,currdeg,seed,draw,incidente,currvadj,currvidx,curreadj );
                if Sixidx(1,3)==0
                    %then acyclic
                    continue;
                end
            end
            current_crossings=Cidx(newM+1)/2;
            incidente=zeros(newN,1);
            for i=1:newM
                if incidente(Sixidx(i,1))==0
                    incidente(Sixidx(i,1))=i;
                end
                if incidente(Sixidx(i,2))==0
                    incidente(Sixidx(i,2))=i;
                end
            end
            [ Final_crossings2,Clabel,Cidx,newM ] = Quick_cross_main_loop( incidente,Sixidx,Clabel,Dlabel,Cidx,newN,newM,currdeg,current_crossings,min_scheme,bigface_depth,Nbeforeembed,verbosity  );
            if Mbeforeembed ~= newM
                disp('undo sub not back to original M 2')
                pause
            end
            Final_crossings=Final_crossings+Final_crossings2;
            
            for j=1:newM
                temph=zeros(Cidx(j+1)-Cidx(j),1);
                temph2=zeros(Cidx(j+1)-Cidx(j),1);
                counth=0;
                for k=Cidx(j)+1:Cidx(j+1)
                    counth=counth+1;
                    temph(counth)=einvmapdel1(einvmaptwo(Clabel(k)));
                    temph2(counth)=einvmapdel1(einvmaptwo(j));
                    fincCcnt(einvmapdel1(einvmaptwo(j))) = fincCcnt(einvmapdel1(einvmaptwo(j))) + 1;
                end
                if(vinvmapdel1(relchk(Sixidx(j,1)))==el(einvmapdel1(einvmaptwo(j)),1) || vinvmapdel1(relchk(Sixidx(j,2)))==el(einvmapdel1(einvmaptwo(j)),2) )
                    %then same dir
                    finClab1(fcnt+1:fcnt+counth)=temph;
                    finCidx1(fcnt+1:fcnt+counth)=temph2;                    
                elseif (vinvmapdel1(relchk(Sixidx(j,2)))==el(einvmapdel1(einvmaptwo(j)),1) || vinvmapdel1(relchk(Sixidx(j,1)))==el(einvmapdel1(einvmaptwo(j)),2) )
                    finClab1(fcnt+1:fcnt+counth)=temph(counth:-1:1);
                    finCidx1(fcnt+1:fcnt+counth)=temph2(counth:-1:1);                                  
                else
                    disp('biconn new map error2');
                    pause
                end
                    fcnt=fcnt+counth;
            end
        else
            
            %newclabel and cidx are just the originals without the pendant
            %edges.           
            t=1;
            vtwo=zeros(origN,1);
            for j=1:N
                vtwo(vinvmapdel1(j))=j;
            end
            
            while t==1
                t=0;
                if N<5
                    break %then k2
                end
                for j=1:N
                    if deglist(j)==2
                        t=1;
                        v1=vadj(vidx(j)+1);
                        v2=vadj(vidx(j)+2);
                        if deglist(v2)>2
                            v1=vadj(vidx(j)+2);
                            v2=vadj(vidx(j)+1);
                        end
                        f2=find(vadj(vidx(v1)+1:vidx(v1+1))==j);
                        if sum(vadj(vidx(v1)+1:vidx(v1+1))==v2)==0
                            vadj(vidx(v1)+f2)=v2;
                            e1 = eadj(vidx(v1)+f2);
                        else
                            %dont reduce because of multi-edge
                            t=0;
                            continue;
                        end
                        f2=find(vadj(vidx(v2)+1:vidx(v2+1))==j);
                        vadj(vidx(v2)+f2)=v1;
                        e2 = eadj(vidx(v2)+f2);
                        eadj(vidx(v2)+f2)=e1;
                        
                        vadj=[vadj(1:vidx(j));vadj(vidx(j+1)+1:end)];
                        eadj=[eadj(1:vidx(j));eadj(vidx(j+1)+1:end)];
                        vidx(j+1:N+1)=[vidx(j+2:N+1)-2;0];
                        deglist(j:N)=[deglist(j+1:N);0];
                        einvmapdel1(e2:M) = [einvmapdel1(e2+1:M);0];
                        vinvmapdel1(j:N) = [vinvmapdel1(j+1:N);0];
                        for k=1:origN
                            if vtwo(k)==j
                                vtwo(k)=v2;
                            end
                        end
                        for k=1:origN
                            if vtwo(k)>j
                                vtwo(k)=vtwo(k)-1;
                            end
                        end
                        for k=1:vidx(N)
                            if vadj(k)>j
                                vadj(k)=vadj(k)-1;
                            end
                            if eadj(k)>e2
                                eadj(k)=eadj(k)-1;
                            end
                        end
                        N=N-1;
                        M=M-1;
                        break
                    end
                end
            end
            if N<5
                continue;
            end
            Sixidx = zeros(M,6);
            for j=1:N
                for k=vidx(j)+1:vidx(j+1)
                    if j<vadj(k)
                        Sixidx(eadj(k),[1 2])=[j,vadj(k)];
                    else
                        Sixidx(eadj(k),[1 2])=[vadj(k),j];
                    end
                end
            end
            incidente=zeros(N,1);
            for i=1:M
                if incidente(Sixidx(i,1))==0
                    incidente(Sixidx(i,1))=i;
                end
                if incidente(Sixidx(i,2))==0
                    incidente(Sixidx(i,2))=i;
                end
            end
            
            if embed_scheme==1
                [Sixidx,Clabel,Dlabel,Cidx] = get_spring_data(Sixidx(:,1:2),deglist,vadj,vidx,0,[],[],seed);
                newM=M;
                newN=N;
            elseif embed_scheme==2
                [Clabel,Cidx,Sixidx,Dlabel ] = embed_circ_2( N,M,seed,Sixidx,vadj,eadj,vidx,draw );
                newM=M;
                newN=N;
                
            elseif embed_scheme==3
                [ Sixidx,incidente,Clabel,Dlabel,Cidx,newN,newM,deglist ]  = embed_plane_build( N,M,Sixidx,deglist,seed,draw,incidente,vadj,vidx,eadj );
                if Sixidx(1,3)==0
                    %then acyclic
                    continue;
                end
            elseif embed_scheme==5
                currx=zeros(N,1);
                curry=zeros(N,1);
                for j=1:N
                    currx(j)=x(vinvmapdel1(j));
                    curry(j)=y(vinvmapdel1(j));
                end
                [Sixidx,Clabel,Dlabel,Cidx] = get_spring_data(Sixidx(:,1:2),deglist,vadj,vidx,1,currx,curry,0);
                newM=M;
                newN=N;
            elseif embed_scheme==6
                [ Sixidx,incidente,Clabel,Dlabel,Cidx,newN,newM,deglist ]  = embed_plane_build_incremental( N,M,Sixidx,deglist,seed,draw,incidente,vadj,vidx,eadj );
                if Sixidx(1,3)==0
                    %then acyclic
                    continue;
                end
            end
            current_crossings=Cidx(newM+1)/2;
            incidente=zeros(newN,1);
            
            for i=1:newM
                if incidente(Sixidx(i,1))==0
                    incidente(Sixidx(i,1))=i;
                end
         
                if incidente(Sixidx(i,2))==0
                    incidente(Sixidx(i,2))=i;
                end
            end

            [ Final_crossings,Clabel,Cidx,M2 ] = Quick_cross_main_loop( incidente,Sixidx,Clabel,Dlabel,Cidx,newN,newM,deglist,current_crossings,min_scheme,bigface_depth,N ,verbosity );
            if M2 ~= M
                disp('undo sub not back to original M 1')
                pause
            end
            for j=1:M
                temph=zeros(Cidx(j+1)-Cidx(j),1);
                temph2=zeros(Cidx(j+1)-Cidx(j),1);
                counth=0;
                for k=Cidx(j)+1:Cidx(j+1)
                    counth=counth+1;
                    temph(counth)=einvmapdel1(Clabel(k));
                    temph2(counth)=einvmapdel1(j);
                    fincCcnt(einvmapdel1(j)) = fincCcnt(einvmapdel1(j)) + 1;
                end
                if vtwo(el(einvmapdel1(j),1))==Sixidx(j,1) && (vtwo(el(einvmapdel1(j),2))==Sixidx(j,2) || Sixidx(j,2)>N)
                    %then same dir
                    finClab1(fcnt+1:fcnt+counth)=temph;
                    finCidx1(fcnt+1:fcnt+counth)=temph2;
                else
                    if vtwo(el(einvmapdel1(j),2))~=Sixidx(j,1) || ( vtwo(el(einvmapdel1(j),1))~=Sixidx(j,2) && Sixidx(j,2)<=N)
                        disp('biconn new map error3');
                        pause
                    end
                    finClab1(fcnt+1:fcnt+counth)=temph(counth:-1:1);
                    finCidx1(fcnt+1:fcnt+counth)=temph2(counth:-1:1);
                end
                fcnt=fcnt+counth;
            end
        end
    end
end

finClab = zeros(fcnt,1);
finCidx = zeros(origM+1,1);
for i=1:origM
    finCidx(i+1) = finCidx(i) + fincCcnt(i);
end
fdn = zeros(origM+1,1);
for i=1:fcnt
    fdn(finCidx1(i)) = fdn(finCidx1(i)) + 1;
    finClab(finCidx(finCidx1(i)) + fdn(finCidx1(i))) = finClab1(i);
end



end

